package Prospector;
use strict;
use DBI;
use Date::Calc qw(:all);
use CGI::Cookie;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);


use Exporter;
$VERSION = 1.00;
@ISA = qw(Exporter);

@EXPORT = qw(&check_cache &get_date &calc_equation &header &get_cookie &check_auth &not_authorized &to_ampm);
@EXPORT_OK = qw(&check_cache &get_date &calc_equation &header &get_cookie &check_auth &not_authorized &to_ampm);
%EXPORT_TAGS = (
	Functions => [qw(&check_cache &get_date &calc_equation &header &get_cookie &check_auth &not_authorized &to_ampm)],
);

my $dbName = "calendar";                # Database Name
my $dbuser = "root";                  # Database User
my $dbpass = "";                      # Password for DB User
my $dType = "mysql";                    # Pg: Postgres, mysql: MySQL
my $connect = "$dbName:localhost";        # dbname=$dbName: Postgres, $dbName:localhost: MySQL

sub header {
	print "Content-type: text/html\n\n";
#	print qq(<META HTTP-EQUIV="expires" CONTENT="0">);
}


sub check_cache {
	(my $userid, my $year) = @_;

	my $dbh = DBI->connect("dbi:$dType:$connect", $dbuser, $dbpass) or die $DBI::errstr;
	my $sth = $dbh->prepare("SELECT holidayid from holiday_cache where sdate>='$year-1-1' and sdate<='$year-12-31' and userid=$userid;");
	$sth->execute;
	my $id = $sth->fetchrow_array;
	$sth->finish;
	$dbh->disconnect;

	create_cache($userid, $year) if(!defined $id || $id eq "");
}

sub create_cache {
	my $userid = $_[0];
	my $year = $_[1];

	my $dbh = DBI->connect("dbi:$dType:$connect", $dbuser, $dbpass) or die $DBI::errstr;
	my $sth = $dbh->prepare("SELECT holidayid, rule, samedate, consecutive from holiday where userid=$userid;");
	$sth->execute;
	while(my @row = $sth->fetchrow_array) {
		my $date = get_date($row[1], $row[2], $year, $userid);
		$dbh->do("INSERT into holiday_cache (holidayid, userid, sdate) VALUES($row[0], $userid, '$year-$date');");

		if(defined $row[3] && $row[3] ne "") {
			(my $month, my $day) = split(/\//, $date);
			for(my $x=1; $x<$row[3]; $x++) {
				(my $newyear,$month,$day) = Add_Delta_Days($year,$month,$day, 1); 
				$dbh->do("INSERT into holiday_cache (holidayid, userid, sdate) VALUES($row[0], $userid, '$newyear-$month-$day');");
			}
		}
	}
	$sth->finish;
	$dbh->disconnect;
}


sub get_date {
	my @number = ("", "First", "Second", "Third", "Fourth", "Last" );
	my @day = ("", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday");
	my @month = ("", "January", "February", "March", "April", "May", "June", "July", "August", "Sepetember", "October", "November", "December");
	my @same = split(/-/, $_[1]);
	my @rules = split(/ /, $_[0]);
	my $year = $_[2];
	my $userid = $_[3];

	if($_[0] eq "* * * *") {
		$same[1] = int $same[1];
		$same[2] = int $same[2];
		return "$same[1]/$same[2]";
	}
	elsif($rules[3] ne "*") {
		return calc_equation($rules[3],$year,$userid);
	}
	else {
		(my $y, my $m, my $d) = Nth_Weekday_of_Month_Year($year,$rules[2],$rules[1],$rules[0]);
		($y, $m, $d) = Nth_Weekday_of_Month_Year($year,$rules[2],$rules[1],4) if($rules[0] eq 5 && !defined $d);
		return "$m/$d";
	}

	return 0;
}


sub calc_equation {
	my $equation = $_[0];
	my $year = $_[1];
	my $easter = 0;
	my @data = 0;
	my $date = 0;
	my $day=0;
	my $month=0;
	my $factor=0;
	my $sth=0;
	my @row=0;
	my $dbh=0;


	if($equation eq "\$easter") {
		return calc_easter($year);
	}
	elsif($equation =~ /-/) {
		@data = split(/-/, $equation);
		$factor = "-$data[1]";

		if($data[0] eq "\$easter") {
			$date = calc_easter($year)."/$year";
			@data = split(/\//, $date);
			($year,$month,$day) = Add_Delta_Days($year,$data[0],$data[1], $factor);
			return "$month/$day";
		}
		else {
			$data[0] =~ s/\$//;

			my $dbh = DBI->connect("dbi:$dType:$connect", $dbuser, $dbpass) or die $DBI::errstr;
			$sth = $dbh->prepare("SELECT rule, samedate from holiday where abbreviation='$data[0]' and userid=$_[2];");
			$sth->execute;
			@row = $sth->fetchrow_array;
			$sth->finish;
			$dbh->disconnect;

			if($row[0] eq "* * * *") {
				@data = split(/-/, $row[1]);
				($year,$month,$day) = Add_Delta_Days($year,$data[0],$data[1], $factor);
				return "$month/$day";
			}
			else {
				@data = split(/ /, $row[0]);
				my $thisyear = $year;

				($year,$month,$day) = Nth_Weekday_of_Month_Year($thisyear,$data[2],$data[1],$data[0]);

	                        if($data[0] eq 5 && !defined $day) {
					($year,$month,$day) = Nth_Weekday_of_Month_Year($thisyear,$data[2],$data[1],4);
                	        }

				($year,$month,$day) = Add_Delta_Days($year,$month,$day,$factor);
				return "$month/$day";
			}
			return "Undefined";
		}
	}
	elsif($equation =~ /\+/) {
		@data = split(/\+/, $equation);
		$factor = $data[1];

		if($data[0] eq "\$easter") {
			$date = calc_easter($year)."/$year";
			@data = split(/\//, $date);
			($year,$month,$day) = Add_Delta_Days($year,$data[0],$data[1], $factor);
			return "$month/$day";
		}
		else {
			$data[0] =~ s/\$//;
			my $dbh = DBI->connect("dbi:$dType:$connect", $dbuser, $dbpass) or die $DBI::errstr;
			$sth = $dbh->prepare("SELECT rule, samedate from holiday where abbreviation='$data[0]' and userid=$_[2];");
			$sth->execute;
			@row = $sth->fetchrow_array;
			$sth->finish;
			$dbh->disconnect;

			if($row[0] eq "* * * *") {
				@data = split(/-/, $row[1]);
				($year,$month,$day) = Add_Delta_Days($year,$data[1],$data[2], $factor);
				return "$month/$day";
			}
			else {
				@data = split(/ /, $row[0]);
				my $thisyear = $year;
				($year,$month,$day) = Nth_Weekday_of_Month_Year($thisyear,$data[2],$data[1],$data[0]);

	                        if($data[0] eq 5 && !defined $day) {
					($year,$month,$day) = Nth_Weekday_of_Month_Year($thisyear,$data[2],$data[1],4);
	                        }

				($year,$month,$day) = Add_Delta_Days($year,$month,$day,$factor);
				return "$month/$day";
			}
			return "Undefined";
		}
	}
	else {
		return "Undefined";
	}

	return "Error!!";
}


sub calc_easter {
	my $year = $_[0];
	my ($year,$month,$day) = Easter_Sunday($year);
	return "$month/$day";
}


sub get_cookie {
	my $get = $_[0];
	my %cookies = fetch CGI::Cookie;

	if(defined %cookies && defined $cookies{$get}) {
		return $cookies{$get}->value;
	}
}

sub check_auth {
	my ($id, $password) = @_;
	return 1 if($password eq "" || $id eq "");

	my $dbh = DBI->connect("dbi:$dType:$connect", $dbuser, $dbpass) or die $DBI::errstr;
	my $sth = $dbh->prepare("SELECT password from users where userid=$id;");
	$sth->execute;
	my $realpw = $sth->fetchrow_array;
	$sth->finish;
	$dbh->disconnect;

	return 0 if($password eq $realpw);
	return 1;
}


sub not_authorized {
	my $query = new CGI;
	print $query->redirect("calendar?action=startup");
	exit 1;
}


sub to_ampm {
	my $time = 0;
	my @data = split(/:/, $_[0]); 

	if($data[0] eq "00" || $data[0] eq "0" || $data[0] eq "24") {
		return "12:$data[1] am";
	} elsif($data[0] > 12) {
		$time = $data[0] - 12;
		return "$time:$data[1] pm";
	} elsif($data[0] == 12) {
		return "$data[0]:$data[1] pm";
	} else {
		return "$data[0]:$data[1] am";
	}
}

1;
